/*
 *  =======================================================================
 *  FoldedBoxArray.java: Create and sort an arraylist of folded box objects.
 *
 *  Written By: Mark Austin                                        May 2007
 *  =======================================================================
 */

package arraylist05;

import java.util.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class FoldedBoxArray {
   ArrayList boxes = new ArrayList();
 
   // Sort boxes by surface area ....

   public void sortBySurfaceArea() {
      Collections.sort( boxes, new areaCompare() );
   }

   class areaCompare implements Comparator {
      public int compare( Object o1, Object o2) {
          FoldedBox b1 = (FoldedBox) o1;
          FoldedBox b2 = (FoldedBox) o2;
          if ( b1.surfaceArea() == b2.surfaceArea() )
               return  0;
          else if ( b1.surfaceArea() > b2.surfaceArea() )
               return  1;
          else
               return -1;
      }
   }
 
   // Sort boxes by volume ....

   public void sortByVolume() {
      Collections.sort( boxes, new volumeCompare() );
   }

   class volumeCompare implements Comparator {
      public int compare( Object o1, Object o2) {
          FoldedBox b1 = (FoldedBox) o1;
          FoldedBox b2 = (FoldedBox) o2;
          if ( b1.volume() == b2.volume() )
               return  0;
          else if ( b1.volume() > b2.volume() )
               return  1;
          else
               return -1;
      }
   }

   // ===========================================================
   // Create an array list of folded box objects ....
   // ===========================================================

   public static void main ( String args [] ) {

      // Create an object of type folded box array ....

      FoldedBoxArray fba = new FoldedBoxArray();

      // Create and initialize folded box objects

      FoldedBox fbMatch = new FoldedBox ("Match",  2.0, 1.0, 0.5 );
      FoldedBox fbShoe  = new FoldedBox ( "Shoe", 12.0, 8.0, 7.0 );
      FoldedBox fbPacking  = new FoldedBox (  "Packing", 36.0, 36.0, 36.0 );
      FoldedBox fbGift     = new FoldedBox(     "Gift", 12.0, 12.0,  5.0 );
      FoldedBox fbDumpster = new FoldedBox( "Dumpster", 72.0, 48.0, 48.0 );

      // Add boxes to the array list .....

      fba.boxes.add( fbMatch );
      fba.boxes.add( fbShoe );
      fba.boxes.add( fbPacking );
      fba.boxes.add( fbGift );
      fba.boxes.add( fbDumpster );

      // Walk along unsorted list and print details ....

      System.out.println("Open folded boxes            ");
      System.out.println("=============================");

      for ( int i = 0; i < fba.boxes.size(); i = i + 1 ) {
          FoldedBox fp = (FoldedBox) fba.boxes.get(i);
          System.out.println( fp.toString() );
      }

      // Sort and print list by volume...

      fba.sortByVolume();

      System.out.println("Folded boxes sorted by volume ");
      System.out.println("============================= ");

      for ( int i = 0; i < fba.boxes.size(); i = i + 1 ) {
          FoldedBox fp = (FoldedBox) fba.boxes.get(i);
          System.out.println( fp.toString() );
      }

      // Sort and print list by surface area...

      fba.sortBySurfaceArea();

      System.out.println("Folded boxes sorted by surface area ");
      System.out.println("=================================== ");

      for ( int i = 0; i < fba.boxes.size(); i = i + 1 ) {
          FoldedBox fp = (FoldedBox) fba.boxes.get(i);
          System.out.println( fp.toString() );
      }

   }
}
