/* 
 *  ==========================================================================
 *  PassengerDataModel.java: Simple data model for storing list of passengers
 *                           on a vehicle. 
 * 
 *  Written by: Mark Austin                                         April 2022 
 *  ==========================================================================
 */ 

package filter04;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import java.lang.Math;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Date;
import java.util.stream.Collectors;

import java.time.LocalDate;

public class PassengerDataModel {
   String dataFileName;
   String  vehicleName;
   boolean DEBUG = true;
   int count = 0;

   double averageAgeSurvivors = 0;
   double averageAgeDied = 0;

   // Arraylist passenger data items ...

   private ArrayList<PassengerDataItem> passengerData;

   // List of passengers who survived and died ...

   private List<PassengerDataItem> survivors;
   private List<PassengerDataItem> died;

   // Hashmap for key = passenger name, value = passenger data items ...

   private HashMap<String,PassengerDataItem> namePassengerDataItemMap;

   // Default constructor method ...

   public PassengerDataModel() {
      passengerData = new ArrayList<PassengerDataItem>();
      namePassengerDataItemMap = new HashMap<String,PassengerDataItem>();
   };

   // Set vehicle name ...

   public void setName( String name ) {
      this.vehicleName = name;
   }

   // ==================================================
   // Load data file into model ...
   // ==================================================

   public void loadDataFile(String dataFileName ) throws IOException {
      this.dataFileName = dataFileName;

      // Open CSVReader and read headers of first row ...

      System.out.printf("--- Read column headers off first row ...\n" );

      CSVReader reader = new CSVReader( new FileReader( dataFileName ));
      String headerLine [] = reader.readNext();

      for (int i = 0; i < headerLine.length; i = i + 1) {
         System.out.printf("Column %d : %s \n", i + 1, headerLine[i] );
      }

      // Loop over passenger list ...

      String [] nextLine;
      while ((nextLine = reader.readNext()) != null) {
         count = count + 1;

         System.out.printf("--- Passenger: %d ...\n", count );

         // Assemble energy data item object ...

         PassengerDataItem item = new PassengerDataItem();
         for (int i = 0; i < headerLine.length; i = i + 1) {
             item.add ( headerLine [i], nextLine[i] );
         }

         // Add passenger data item to array list ...

         passengerData.add (item );

         // Add passenger data item to hashmap ...

         String passengerName = nextLine[2];
         namePassengerDataItemMap.put( passengerName, item );

         System.out.println( item.toString() );
      }
   }

   // =================================================
   // Process populated data model ...
   // 
   // =================================================

   public void process() {

      System.out.printf("--- Enter process() ...\n");

      // Assemble lists of passengers who survived and died ....

      survivors = passengerData.stream().filter( p -> p.getPassengerSurvived() == true)
                                        .collect(Collectors.toList());

      died = passengerData.stream().filter( p -> p.getPassengerSurvived() == false)
                                   .collect(Collectors.toList());

      // Compute average age of survivors and passengers who died ...

      double sum = 0;
      for ( int i = 0; i < survivors.size(); i = i + 1 ) {
          PassengerDataItem item = (PassengerDataItem) survivors.get(i);
          sum = sum + item.getAge();
      }

      averageAgeSurvivors = sum/survivors.size();

      sum = 0;
      for ( int i = 0; i < died.size(); i = i + 1 ) {
          PassengerDataItem item = (PassengerDataItem) died.get(i);
          sum = sum + item.getAge();
      }

      averageAgeDied = sum/died.size();

      System.out.printf("--- Leave process() ...\n");
   }

   // =================================================
   // String description of passenger data model ...
   // =================================================

   public String toString() {
      StringBuffer buf = new StringBuffer();

      buf.append( String.format("PassengerDataModel ... \n"));
      buf.append( String.format("======================================= ... \n"));
      buf.append( String.format("--- Data file: %s ... \n", dataFileName ));
      buf.append( String.format("--- Passenger Vehicle: %s ... \n", vehicleName ));
      buf.append( String.format("--- No passengers: %d ... \n", count ));
      buf.append( String.format("--- No survivors: %d ... \n", survivors.size() ));
      buf.append( String.format("--- No deaths: %d ... \n", died.size() ));
      buf.append( String.format("--- Average age (survivors): %.1f ... \n", averageAgeSurvivors ));
      buf.append( String.format("--- Average age (died): %.1f ... \n", averageAgeDied ));
      buf.append( String.format("======================================= ... \n"));

      return buf.toString();
   }
}
