/*
 * ===============================================================
 * WayPoint.java: Create waypoint in the polyline system.
 * 
 * -- A waypoint records its coordinate, its distance from the
 *    coordinate source, and a reference to a set of features
 *    if they exist.
 * 
 * Written by: Mark Austin                           November 2019
 * ===============================================================
 */

package geometry;

public class WayPoint extends Node2D { 
    protected double distanceFromSource      = 0.0;
    protected double distanceFromDestination = 0.0;
    protected double elevation = 0.0;

    // Contructor methods ...
    
    public WayPoint() {
       super();
    }

    public WayPoint( double dX, double dY ) {
       super( dX, dY );
    }

    // -----------------------------------------------------------
    // Set/get distance from source/destination ....
    // -----------------------------------------------------------

    public void setDistanceFromSource ( double distance ) {
       this.distanceFromSource = distance;
    }

    public double getDistanceFromSource () {
       return distanceFromSource;
    }

    public void setDistanceFromDestination ( double distance ) {
       this.distanceFromDestination = distance;
    }

    public double getDistanceFromDestination () {
       return distanceFromDestination;
    }

    // -----------------------------------------------------------
    // Create string description of waypoint ...
    // -----------------------------------------------------------

    public String toString() {
       StringBuffer buf = new StringBuffer();

       buf.append(String.format("WayPoint(%s) ...\n", getName() ));
       buf.append(String.format("--- (x,y) = (%.3f, %.3f) ...\n", getX(), getY() ));
       buf.append(String.format("--- distance from source      = %.2f ...\n", distanceFromSource ));
       buf.append(String.format("--- distance from destination = %.2f ...\n", distanceFromDestination ));

       return buf.toString();
   }
}
